const chalk = require('chalk');
const yargs = require('yargs');

class CommandBase {
    constructor({ command, desc, handler, builder }) {
        this.command = command;
        this.desc = desc;
        this.handler = handler;
        this.builder = builder;
    }

    get yargs() {
        return {
            command: this.command,
            desc: this.desc,
            handler: async (argv) => {
                try {
                    await this.handler(argv);
                } catch (e) {
                    console.error(e);
                    console.log(chalk.red(`Error! ${e.message}\n`));
                    yargs.showHelp();
                }
            },
            builder: this.builder,
        };
    }
}

module.exports = CommandBase;
