const { Codefresh, Config } = require('codefresh-sdk');
const _ = require('lodash');

class TSLogic {
    static async getTsFile(url) {
        const sdk = new Codefresh();

        sdk.configure(await Config.nonAuthenticated({
            url,
        }));

        const sdkData = this.#sdkTSparser(sdk);

        return 'import SdkHolderBase from \'./sdk.holder.base\'\n' + sdkData.join('\n\n') + '\nexport default SdkHolderBase';
    }


    static #sdkTSparser = (sdk) => {
        const methods = {};

        const result = [];

        const scanner = (path = []) => {
            const interfaceObjects = [];
            const methodsList = [];

            const objPath = path.join('.');
            _.forEach(_.get(sdk, objPath) || sdk, (value, key) => {
                if (_.isFunction(value)) {
                    const methodsPath = path.concat('_methods').join('.');
                    _.set(methods, methodsPath, (_.get(methods, methodsPath, [])).concat(key));
                    methodsList.push(key);
                    return;
                }

                if (_.isPlainObject(value)) {
                    scanner(path.concat(key));
                    interfaceObjects.push({
                        name: key,
                        interface: this.#generateName(path.concat(key)),
                    });
                }
            });

            const itemName = this.#generateName(path);
            result.push(this.#generateItem(itemName ? itemName : 'sdk', methodsList, interfaceObjects, 'interface', !itemName));
        };

        scanner();

        return result;
    };

    static #generateName = (path) => {
        if (!path.length) {
            return null;
        }
        return path.map(item => _.upperFirst(_.toLower(item))).join('');
    };

    /**
     *
     * @param name
     * @param methods
     * @param {{name: String, interface: String}[]} interfaces
     * @param {('interface'|'class')} type
     * @param needExport
     * @return {string}
     */
    static #generateItem = (name, methods = [], interfaces = [], type = 'interface', needExport = false) => {
        const methodsAdapted = methods
            .map(item => `'${item}'()`)
            .join('\n');

        const interfaceAdapted = interfaces
            .map(item => `'${item.name}': ${item.interface}`)
            .join('\n');

        return `${needExport ? 'export ' : ''}${type} ${name} {
                    ${methodsAdapted}
                    ${interfaceAdapted}
                }`;
    };

    static #declareModule = (str) => {
        return 'declare module \'codefresh-sdk\' {\n' + str + '\n}';
    };
}

module.exports = TSLogic;
