const fs = require('fs').promises;
const path = require('path');
const _ = require('lodash');

class ListTagLogic {

    static #TAG_REGEXP = /@([a-zA-Z0-9-]+)/g;

    static async getTags() {
        let result = new Set();

        const fileScanner = async (dirPath = '../../../features') => {
            const fileListing = await fs.readdir(path.resolve(__dirname, dirPath));

            let promises = [];

            for (let item of fileListing) {
                const currentFilePath = path.resolve(__dirname, dirPath + '/' + item);
                const lstatInfo = await fs.lstat(currentFilePath);
                if (lstatInfo.isDirectory()) {
                    promises.push(fileScanner(dirPath + '/' + item));
                    continue;
                }

                if (item.endsWith('.feature')) {
                    const fileContent = await fs.readFile(currentFilePath, 'utf8');
                    const matchedTags = (fileContent.match(this.#TAG_REGEXP) || []).map(item => item.trim());
                    _.forEach(matchedTags, item => result.add(item));
                }
            }

            await Promise.all(promises);
        };

        await fileScanner();

        return [...result];
    }
}

module.exports = ListTagLogic;
