const axios = require('axios');

class Api {

    #token = null;
    #endpoint = null;

    constructor({ endpoint, token }) {
        this.#endpoint = endpoint;
        this.setAuthToken(token);
    }


    setAuthToken(token) {
        this.#token = token;
    }

    createAccount(name, provider = 'github') {
        return this.#call({
            method: 'POST',
            path: '/admin/accounts',
            body: {
                name,
                provider,
            },
        });
    }

    addUserToAccount(accountId, mail, provider = 'github') {
        return this.#call({
            method: 'POST',
            path: `/accounts/${accountId}/adduser`,
            body: {
                userDetails: mail,
                provider: {
                    name: provider,
                },
                roles: [
                    'User', 'Admin'
                ]
            },
        });
    }

    async getAccessTokenByUserId(userId) {
        const { accessToken } = await this.#call({
            method: 'GET',
            path: `/admin/user/loginAsUser`,
            qs: {
                userId,
            },
        });

        return accessToken;
    }

    activeUser(userId) {
        return this.#call({
            method: 'POST',
            path: `/admin/user/${userId}/activate`,
        });
    }

    markAsAdmin(accountId, userId) {
        return this.#call({
            method: 'POST',
            path: `/accounts/${accountId}/${userId}/admin`,
        });
    }


    deleteUser(userName) {
        return this.#call({
            method: 'DELETE',
            path: `/admin/user/${userName}`
        })
    }

    deleteAccount(accountId) {
        return this.#call({
            method: 'DELETE',
            path: `/admin/accounts/${accountId}`
        })
    }

    async getUserByUserMail(userName) {
        const result = await this.#call({
            method: 'GET',
            path: `/admin/user`,
            qs: {
                'filter[userName]': userName,
            },
        });

        return result[0];
    }

    getUser() {
        return this.#call({
            method: 'GET',
            path: `/user`
        });
    }

    #getAuthToken = () => {
        if (!this.#token) {
            return {};
        }

        return {
            authorization: this.#token,
        };
    };

    #call = async ({ method, qs, body, path, headers }) => {
        const result = await axios({
            url: `${this.#endpoint}/api${path}`,
            headers: {
                ...headers || {},
                ...this.#getAuthToken(),
            },
            data: body,
            params: qs,
            method: method || 'GET',
        });

        return result.data;
    };
}

module.exports = Api;
