const { Codefresh, Config } = require('codefresh-sdk');

const _ = require('lodash');

class SdkHolder {
    static sdk = {};
    static #usedMethods = new Set();
    static allMethods = [];
    static #defaultUser = {};

    static getDefaultUser () {
        return SdkHolder.#defaultUser;
    }

    static getDefaultAccount () {
        return SdkHolder.#defaultUser.account[0];
    }

    static get usedMethods() {
        return [...this.#usedMethods];
    }

    static #methodHandler = (f, path) => {
        return (...args) => {
            this.#usedMethods.add(path);
            return f.apply(null, args);
        };
    };

    static async initSdk({ context, baseEndpoint, userToken }) {
        const sdk = new Codefresh();

        if (context === 'true') {
            sdk.configure(await Config.fromCodefreshConfig());
        } else {
            sdk.configure(await Config.load({
                url: baseEndpoint,
                apiKey: userToken,
            }));
        }


        const data = this.#sdkScanner(sdk);
        this.allMethods = data;

        _.forEach(data, item => {
            _.set(this.sdk, item, this.#methodHandler(_.get(sdk, item), item));
        });

        SdkHolder.#defaultUser = await this.sdk.users.get();
    }

    static #sdkScanner = (sdk) => {
        const methods = [];

        const scanner = (path = []) => {
            _.forEach(_.get(sdk, path.join('.')) || sdk, (value, key) => {
                if (_.isFunction(value)) {
                    return methods.push(path.concat(key).join('.'));
                }

                if (_.isPlainObject(value)) {
                    scanner(path.concat(key));
                }
            });
        };

        scanner();

        return methods;
    };
}

module.exports = SdkHolder;
